package quimufu.colourful_portals.mixin;

import com.llamalad7.mixinextras.sugar.Local;
import net.minecraft.block.BlockState;
import net.minecraft.entity.Entity;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.RaycastContext;
import net.minecraft.world.World;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import quimufu.colourful_portals.util.CollisionAwareShapeContext;

import java.util.HashSet;

import static quimufu.colourful_portals.ColourfulPortalsMod.PORTAL_BLOCK;

@Mixin(Entity.class)
public class EntityMixin {
    @Shadow
    private World world;

    @Shadow
    public double prevX;

    @Shadow
    public double prevY;

    @Shadow
    public double prevZ;

    @Shadow
    private Vec3d pos;

    @Unique
    private final ThreadLocal<HashSet<BlockPos>> calledAlready = ThreadLocal.withInitial(HashSet::new);

    @Inject(at = @At(value = "INVOKE", target = "Lnet/minecraft/block/BlockState;onEntityCollision(Lnet/minecraft/world/World;Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/entity/Entity;)V"), method = "checkBlockCollision")
    public void cpm_dontCallTwice(CallbackInfo ci, @Local BlockPos.Mutable mutable) {
        calledAlready.get().add(mutable.toImmutable());
    }

    @Inject(at = @At("RETURN"), method = "checkBlockCollision")
    public void cpm_afterCheckBlockCollision(CallbackInfo ci) {
        if (!world.isClient()) {
            BlockHitResult hitResult = world.raycast(new RaycastContext(new Vec3d(prevX, prevY, prevZ), pos, RaycastContext.ShapeType.COLLIDER, RaycastContext.FluidHandling.NONE, new CollisionAwareShapeContext()));

            BlockPos hitResultBlockPos = hitResult.getBlockPos();
            if (!calledAlready.get().contains(hitResultBlockPos)) {
                BlockState state = world.getBlockState(hitResultBlockPos);
                if (state.getBlock() == PORTAL_BLOCK) {
                    state.onEntityCollision(world, hitResultBlockPos, (Entity) ((Object) this));
                }

            }
            calledAlready.get().clear();
        }

    }
}
